<?php

include_once dirname(__FILE__). "/../../config/config.php";
use phpseclib3\Crypt\PublicKeyLoader;

class PaypayCardCommunicatePaymentModuleFrontController extends ModuleFrontController
{

    /**
     * [initContent this method is load on file is call]
     */
    public function initContent()
    {
        if (!$this->context->customer->isLogged(true) ||
            empty($this->context->cart) ||
            $this->context->currency->name != 'Euro') {
            Tools::redirect('index.php');
        }

        $this->display_column_left  = false;
        $this->display_column_right = false;

        parent::initContent();

        $RSAKey              = PublicKeyLoader::load(file_get_contents(_PS_UPLOAD_DIR_ . $this->module->name . "/RSAKey.pem"));
        $PayPayEncryptionKey = $RSAKey->decrypt(base64_decode(Configuration::get('PAYPAY_ENCRYPTION_KEY_CARD')));
        $PayPayPlataformCode = Configuration::get('PAYPAY_PLATAFORM_CODE_CARD');
        $PayPayNIF           = Configuration::get('PAYPAY_NIF_CARD');
        $paymentMethod       = Tools::getValue('paypay_method');

        $classmap = array(
            'RequestEntity'                     => 'CardRequestEntity',
            'ResponseIntegrationState'          => 'CardResponseIntegrationState',
            'RequestInterval'                   => 'CardRequestInterval',
            'RequestPaymentReference'           => 'CardRequestPaymentReference',
            'ResponseEntityPayments'            => 'CardResponseEntityPayments',
            'ResponseEntityPaymentReferences'   => 'CardResponseEntityPaymentReferences',
            'RequestBillingAddress'             => 'CardRequestBillingAddress',
            'RequestShippingAddress'            => 'CardRequestShippingAddress',
            'RequestCreditCardPayment'          => 'CardRequestCreditCardPayment',
            'ResponseCreditCardPayment'         => 'CardResponseCreditCardPayment',
            'RequestReferenceDetails'           => 'CardRequestReferenceDetails',
            'ResponseGetPayment'                => 'CardResponseGetPayment',
            'RequestPaymentOrder'               => 'CardRequestPaymentOrder',
            'RequestWebhook'                    => 'CardRequestWebhook',
            'RequestPayment'                    => 'CardRequestPayment',
            'RequestPaymentOption'              => 'CardRequestPaymentOption',
        );

        $options = array(
            'classmap' => $classmap,
            'location' => defined('PAYPAY_SERVER_URL') ? constant('PAYPAY_SERVER_URL') : Configuration::get("WEBSERVICEURL_CARD"),
            'cache_wsdl' => WSDL_CACHE_NONE
        );

        if (!$this->completeOrder($PayPayEncryptionKey, $PayPayPlataformCode, $PayPayNIF, $options, $paymentMethod)) {
            $this->setTemplate('error.tpl');
        }
    }

    /**
     * [completeOrder communicate order to PayPay plataform]
     * @param  [string] $PayPayEncryptionKey [encryption key for the PayPay webservice]
     * @param  [string] $PayPayPlataformCode [plataform code for the PayPay webservice]
     * @param  [string] $PayPayNIF           [nif for the PayPay webservice]
     * @param  [array] $options              [options for the PayPay webservice]
     * @param  [string] $code                [code of payment option]
     * @return [boolean]                     [success operation]
     */
    private function completeOrder($PayPayEncryptionKey, $PayPayPlataformCode, $PayPayNIF, $options, $paymentMethod)
    {
        try {
            $cart                     = $this->context->cart;
            $customer                 = new Customer($cart->id_customer);
            $billing                  = new Address($cart->id_address_invoice);
            $shipping                 = new Address($cart->id_address_delivery);
            $dateTime                 = new DateTime();
            $dateTime                 = $dateTime->modify('+1 minutes');
            $dateTime                 = $dateTime->format('d-m-Y H:i:s');

            $cancelLink               = $this->context->link->getModuleLink('paypayCard', 'cancelCreditCardPayment', array('cartId' => $cart->id));
            $successPaymentLink       = $this->context->link->getModuleLink('paypayCard', 'successPayment', array('cartId'=>$cart->id));
            $returnLink               = $this->context->link->getModuleLink('paypayCard', 'backPayment', array('cartId'=>$cart->id));

            $client                    = new SoapClient(defined('PAYPAY_WSDL_URL') ? constant('PAYPAY_WSDL_URL') : Configuration::get("WEBSERVICEWSDL_CARD"), $options);
            $hash                      = hash("sha256", $PayPayEncryptionKey.$dateTime);
            $lang                      = $this->context->language->iso_code;
            $info                      = array(
                                            "Prestashop" => constant('_PS_VERSION_')
                                        );

            $entity                    = new CardRequestEntity($PayPayPlataformCode, $hash, $dateTime, $PayPayNIF, $lang, $info);
            $amountDecimal             = $cart->getOrderTotal(true, Cart::BOTH);
            $amount                    = number_format($amountDecimal, 2, '', '');
            $amountFormatted           = number_format($amountDecimal, 2);

            //Get ISO 3166 2 digit country codes from Billing Address
            $countryIsoBilling = Country::getIsoById($billing->id_country);
            //Get ISO 3166 2 digit country codes from Shipping Address
            $countryIsoShipping = Country::getIsoById($shipping->id_country);

            //Get the name of the State
            $billingStateName = State::getNameById($billing->id_state);

            //Get the ISO Code from the Billing State
            $sqlB = "SELECT iso_code
                        FROM " . _DB_PREFIX_ . "state
                        WHERE id_state = " . (int) $billing->id_state;
            $billingStateIso = Db::getInstance()->getValue($sqlB);

            //Get the name of the State
            $shippingStateName = State::getNameById($shipping->id_state);

            //Get the ISO Code from the shipping State
            $sqlS = "SELECT iso_code
                        FROM " . _DB_PREFIX_ . "state
                        WHERE id_state = " . (int) $shipping->id_state;
            $shippingStateIso = Db::getInstance()->getValue($sqlS);


            //validate and concluded the order
            $this->module->validateOrder(
                (int)$cart->id,
                Configuration::get("PAYPAY_STATUS_0"),
                (float)$amountDecimal,
                'paypay_card',
                null,
                array(),
                null,
                false,
                $customer->secure_key
            );

            //get the PrestaShop reference for the order
            $sql = "SELECT reference
                        FROM " . _DB_PREFIX_ . "orders
                        WHERE id_order = '" . (int)$this->module->currentOrder . "'";
            if ($row = Db::getInstance()->getRow($sql)) {
                $referenceOrder = $row['reference'];
            }

            $order = new CardRequestPaymentOrder(array('amount'=>(int)$amount, 'productCode'=> $referenceOrder));
            $cliente = new CardRequestBuyerInfo(
                array(
                    'lastName'   => $this->context->customer->lastname,
                    'firstName'  => $this->context->customer->firstname,
                    'customerId' => $this->context->customer->id,
                    'email'      => $this->context->customer->email
                )
            );
            $billingAddress = new CardRequestBillingAddress(
                $countryIsoBilling,
                $billingStateIso,
                $billingStateName,
                $billing->city,
                $billing->address1,
                $billing->address2,
                $billing->postcode
            );
            $shippingAddress = new CardRequestShippingAddress(
                $countryIsoShipping,
                $shippingStateIso,
                $shippingStateName,
                $shipping->city,
                $shipping->address1,
                $shipping->address2,
                $shipping->postcode
            );

            $payment = new CardRequestCreditCardPayment(
                $order,
                $successPaymentLink,
                $cancelLink,
                $returnLink,
                $cliente,
                $paymentMethod['code'],
                $billingAddress,
                $shippingAddress
            );

            $paymentDetails  = $client->doWebPayment($entity, $payment);

            //check sucess of the request
            if ($paymentDetails) {
                //Add order in the paypay table
                $hash = md5(date('Y-m-d H:i:s'));
                $idPaymentPaypay = $paymentDetails->paymentId;

                Db::getInstance()->insert('paypay_payments', array(
                    'idPaymentPaypay' => (int) $idPaymentPaypay,
                    'idOrder'         => (int) $this->module->currentOrder,
                    'paypayLink'      => isset($paymentDetails->url) ? pSQL($paymentDetails->url) : null,
                    'token'           => isset($paymentDetails->token) ? pSQL($paymentDetails->token) : null,
                    'amount'          => $amountFormatted,
                ));

                //Email generation to send a client
                $templateVars = array(
                    'method'         => $paymentMethod['name'],
                    'paypayLink'     => isset($paymentDetails->url) ? ($paymentDetails->url) : null,
                    'order_name'     => $referenceOrder,
                    'firstname'      => $customer->firstname,
                    'lastname'       => $customer->lastname,
                );

                $psOrder = new Order($this->module->currentOrder); //order with id=1
                $psOrder->addOrderPayment((float)$amountDecimal, null, $idPaymentPaypay);


                Mail::Send(
                    (int)$this->context->language->id,
                    'info_order',
                    Mail::l($this->module->l('A aguardar pagamento'), $this->context->language->id),
                    $templateVars,
                    $customer->email
                );

                Tools::redirect($paymentDetails->url);
            } else {
                return false;
            }
        } catch (SoapFault $error) {
            $error = 0;
            return $error;
        }
    }
}
