<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Luís Gonçalves
 *  @copyright 2015-2016 PayPay
 *  @license   LICENSE.txt
 */

include_once dirname(__FILE__). "/../../config/config.php";
use phpseclib3\Crypt\PublicKeyLoader;

class PaypayMultibancoUpdatePaymentWebhookModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        $RSAKey              = PublicKeyLoader::load(file_get_contents(_PS_UPLOAD_DIR_ . $this->module->name . "/RSAKey.pem"));
        $PayPayEncryptionKey = $RSAKey->decrypt(base64_decode(Configuration::get('PAYPAY_ENCRYPTION_KEY_MULTIBANCO')));
        $hookHash            = Tools::getValue('hookHash');
        $hookDate            = Tools::getValue('hookDate');
        $hookAction          = Tools::getValue('hookAction');
        $payments            = Tools::getValue('payments');
        $body                = file_get_contents("php://input");
        $signature           = str_replace(',', '&', $_SERVER['HTTP_PAYPAY_SIGNATURE']);

        parse_str($signature, $output);

        if ((int)($output['ts']) > (time() - 1800) && hash_equals($output['hash'], hash_hmac('sha256', $body.$output['ts'], $PayPayEncryptionKey))) {
            $idStatusPaid = 2;

            foreach ($payments as $payment) {
                $orderDetails = $this->getOrderDetails($payment['paymentId']);

                if ($orderDetails) {
                    $this->updateOrderHistoryPaymentState($orderDetails['idOrder'], $idStatusPaid);
                }
            }
        }
        if (version_compare(Tools::substr(_PS_VERSION_, 0, 3), '1.7', '>=')) {
            $this->setTemplate('module:paypayMultibanco/views/templates/front/empty.tpl');
        }
    }



    /**
     * [getOrderDetails gives the order  PrestaShop by id Payment from PayPay plataform]
     * @param  [int] $idPaymentPaypay [id Payment from PayPay plataform]
     * @return [int]                  [PrestaShop order id ]
     */
    private function getOrderDetails($idPaymentPaypay)
    {
        //get all orders waiting payments
        $sql = "SELECT pp.idOrder,
                       pp.idPaymentPaypay,
                       pp.reference,
                       op.id_order_payment,
                       o.total_paid_tax_excl,
                       o.total_paid_tax_incl,
                       o.total_shipping_tax_excl,
                       o.total_shipping_tax_incl,
                       o.total_products,
                       o.total_products_wt
                FROM "._DB_PREFIX_."paypay_payments pp
                LEFT JOIN "._DB_PREFIX_."order_history oh ON oh.id_order = pp.idOrder
                LEFT JOIN "._DB_PREFIX_."orders o ON o.id_order = pp.idOrder
                LEFT JOIN "._DB_PREFIX_."order_payment op ON op.order_reference = o.reference
                WHERE oh.id_order_state = '".(int)Configuration::get('PAYPAY_STATUS_0')."'
                      AND pp.idPaymentPaypay = '".(int)$idPaymentPaypay."'";

        if ($row = Db::getInstance()->getRow($sql)) {
            return array('idOrder'                 => $row['idOrder'],
                         'id_order_payment'        => $row['id_order_payment'],
                         'total_paid_tax_excl'     => $row['total_paid_tax_excl'],
                         'total_paid_tax_incl'     => $row['total_paid_tax_incl'],
                         'total_shipping_tax_excl' => $row['total_shipping_tax_excl'],
                         'total_shipping_tax_incl' => $row['total_shipping_tax_incl'],
                         'total_products'          => $row['total_products'],
                         'total_products_wt'       => $row['total_products_wt']
                         );
        } else {
            return 0;
        }
    }

    /**
     * [updateOrderHistoryPaymentState updates payments states orders in history payments]
     * @param  [int] $idOrder          [order id]
     * @param  [int] $idStatusPaid     [is status payment ]
     * @return void
     */
    private function updateOrderHistoryPaymentState($idOrder, $idStatusPaid)
    {
        require_once _PS_ROOT_DIR_ . '/config/config.inc.php';
        require_once _PS_CLASS_DIR_ . 'order/Order.php';
        require_once _PS_CLASS_DIR_ . 'order/OrderHistory.php';

        $objOrder = new Order($idOrder);

        if ($objOrder->hasBeenPaid()) {
            return 0;
        }

        $objOrder->setCurrentState($idStatusPaid); //order with id=1
    }
}
