<?php
include_once dirname(__FILE__). "/../../config/config.php";
use phpseclib3\Crypt\PublicKeyLoader;

class PaypayMultibancoUpdateStatePaymentsModuleFrontController extends ModuleFrontController
{

    /**
     * [initContent   this method is load on file is call]
     */
    public function initContent()
    {
        $hash = Tools::getValue('hash');
        $confHash = Configuration::get('HASH_MULTIBANCO');

        if ($hash != $confHash) {
            Tools::redirect('index.php');
        }

        parent::initContent();

        $RSAKey              = PublicKeyLoader::load(file_get_contents(_PS_UPLOAD_DIR_ . $this->module->name . "/RSAKey.pem"));
        $PayPayEncryptionKey = $RSAKey->decrypt(base64_decode(Configuration::get('PAYPAY_ENCRYPTION_KEY_MULTIBANCO')));
        $PayPayPlataformCode = Configuration::get('PAYPAY_PLATAFORM_CODE_MULTIBANCO');
        $PayPayNIF           = Configuration::get('PAYPAY_NIF_MULTIBANCO');

        $classmap = array(
            'RequestEntity'                     => 'MultibancoRequestEntity',
            'ResponseIntegrationState'          => 'MultibancoResponseIntegrationState',
            'RequestInterval'                   => 'MultibancoRequestInterval',
            'RequestPaymentReference'           => 'MultibancoRequestPaymentReference',
            'ResponseEntityPayments'            => 'MultibancoResponseEntityPayments',
            'ResponseEntityPaymentReferences'   => 'MultibancoResponseEntityPaymentReferences',
            'RequestBillingAddress'             => 'MultibancoRequestBillingAddress',
            'RequestShippingAddress'            => 'MultibancoRequestShippingAddress',
            'RequestCreditCardPayment'          => 'MultibancoRequestCreditCardPayment',
            'ResponseCreditCardPayment'         => 'MultibancoResponseCreditCardPayment',
            'RequestReferenceDetails'           => 'MultibancoRequestReferenceDetails',
            'ResponseGetPayment'                => 'MultibancoResponseGetPayment',
            'RequestPaymentOrder'               => 'MultibancoRequestPaymentOrder',
            'RequestWebhook'                    => 'MultibancoRequestWebhook',
            'RequestPayment'                    => 'MultibancoRequestPayment',
            'RequestPaymentOption'              => 'MultibancoRequestPaymentOption',
        );

        $options = array(
            'classmap' => $classmap,
            'location' => defined('PAYPAY_SERVER_URL') ? constant('PAYPAY_SERVER_URL') : Configuration::get("WEBSERVICEURL_MULTIBANCO"),
            'cache_wsdl' => WSDL_CACHE_NONE
        );

        if (!$this->updateStatePayments($PayPayEncryptionKey, $PayPayPlataformCode, $PayPayNIF, $options)) {
            $this->setTemplate('error.tpl');
        } else {
            $link = strtok($_SERVER['HTTP_REFERER'], '?');
            Tools::redirectAdmin($link.'?controller=AdminOrders');
        }
    }

    /**
     * [updateStatePayments  update state of the payment]
     * @param  [string] $PayPayEncryptionKey [encryption key for the PayPay webservice]
     * @param  [string] $PayPayPlataformCode [plataform code for the PayPay webservice]
     * @param  [string] $PayPayNIF           [nif for the PayPay webservice]
     * @param  [string] $options             [options for the PayPay webservice]
     * @return [boolean]                     [success operation]
     */
    private function updateStatePayments($PayPayEncryptionKey, $PayPayPlataformCode, $PayPayNIF, $options)
    {
        $allIdsorders          = array();
        $allIdsPaymentPayPay   = array();
        $requestEntityPayments = new MultibancoRequestEntityPayments();

        //get all orders waiting payments
        $sql = "SELECT pp.idOrder, pp.idPaymentPaypay, pp.reference
                FROM "._DB_PREFIX_."paypay_payments pp
                LEFT JOIN "._DB_PREFIX_."order_history oh ON oh.id_order = pp.idOrder
                WHERE oh.id_order_state = '".(int)Configuration::get('PAYPAY_STATUS_0')."'";

        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                $RequestReferenceDetails = new MultibancoRequestReferenceDetails(array('reference'=>$row['reference'], 'paymentId'=>$row['idPaymentPaypay']));
                array_push($requestEntityPayments->payments, $RequestReferenceDetails);
                array_push($allIdsorders, $row['idOrder']);
                array_push($allIdsPaymentPayPay, $row['idPaymentPaypay']);
            }
        }

        //communicate PayPay webservice and check state for orders
        try {
            $dateTime = new DateTime();
            $dateTime = $dateTime->modify('+1 minutes');
            $dateTime = $dateTime->format('d-m-Y H:i:s');

            $client = new SoapClient(defined('PAYPAY_WSDL_URL') ? constant('PAYPAY_WSDL_URL') : Configuration::get("WEBSERVICEWSDL_MULTIBANCO"), $options);
            $hash   = hash("sha256", $PayPayEncryptionKey.$dateTime);
            $entity = new MultibancoRequestEntity($PayPayPlataformCode, $hash, $dateTime, $PayPayNIF, "PT", []);

            $paymentsRequest = $client->checkEntityPayments($entity, $requestEntityPayments);

            $allStateOrders  = array();
            foreach ($paymentsRequest->payments as $paymentStateRequest) {
                array_push($allStateOrders, $paymentStateRequest->paymentState);
            }

            $this->updateOrderHistoryPaymentState($allIdsorders, $allStateOrders, $allIdsPaymentPayPay);

            return true;
        } catch (SoapFault $error) {
            $error = 0;
            return $error;
        }
    }

    /**
     * [getOrderDetails gives the order  PrestaShop by id Payment from PayPay plataform]
     * @param  [int] $idPaymentPaypay [id Payment from PayPay plataform]
     * @return [int]                  [PrestaShop order id ]
     */
    private function getOrderDetails($allIdsPaymentPayPay)
    {
        $allOrderDetails = array();
        foreach ($allIdsPaymentPayPay as $idPaymentPaypay) {

            //get all orders waiting payments
            $sql = "SELECT pp.idOrder,
                           pp.idPaymentPaypay,
                           pp.reference,
                           op.id_order_payment,
                           o.total_paid_tax_excl,
                           o.total_paid_tax_incl,
                           o.total_shipping_tax_excl,
                           o.total_shipping_tax_incl,
                           o.total_products,
                           o.total_products_wt
                    FROM "._DB_PREFIX_."paypay_payments pp
                    LEFT JOIN "._DB_PREFIX_."order_history oh ON oh.id_order = pp.idOrder
                    LEFT JOIN "._DB_PREFIX_."orders o ON o.id_order = pp.idOrder
                    LEFT JOIN "._DB_PREFIX_."order_payment op ON op.order_reference = o.reference
                    WHERE oh.id_order_state = '".(int)Configuration::get('PAYPAY_STATUS_0')."'
                          AND pp.idPaymentPaypay = '".(int)$idPaymentPaypay."'";

            if ($row = Db::getInstance()->getRow($sql)) {
                array_push($allOrderDetails, array( 'idOrder'                 => $row['idOrder'],
                                                    'id_order_payment'          => $row['id_order_payment'],
                                                    'total_paid_tax_excl'     => $row['total_paid_tax_excl'],
                                                    'total_paid_tax_incl'     => $row['total_paid_tax_incl'],
                                                    'total_shipping_tax_excl' => $row['total_shipping_tax_excl'],
                                                    'total_shipping_tax_incl' => $row['total_shipping_tax_incl'],
                                                    'total_products'          => $row['total_products'],
                                                    'total_products_wt'       => $row['total_products_wt']
                             ));
            }
        }
        return $allOrderDetails;
    }

    /**
     * [updateOrderHistoryPaymentState updates payments states orders in history payments]
     * @param  [array] $allIdsorders   [array with all orders still waiting payment]
     * @param  [array] $allStateOrders [array with all status for all orders still waiting payment ]
     * @param  [array] $allIdsPaymentPayPay [array with all ids from PayPay platform ]
     * @param  [int] $idPay            [id for paid status]
     * @return void
     */
    private function updateOrderHistoryPaymentState($allIdsorders, $allStateOrders, $allIdsPaymentPayPay)
    {
        $idStatusPaid = 2;

        for ($i=0; $i < sizeof($allIdsorders); $i++) {
            if ($allStateOrders[$i] == 1) { //Paid
                /*                $sql = "UPDATE "._DB_PREFIX_."order_history oh
                                        SET oh.id_order_state = '".(int)$idStatusPaid."'
                                        WHERE oh.id_order = '".(int)$allIdsorders[$i]."' ";

                                Db::getInstance()->ExecuteS($sql);

                                $allOrderDetails = $this->getOrderDetails($allIdsPaymentPayPay);
                                foreach ($allOrderDetails as $allOrderDetail) {
                                    $orderInvoiceId = $this->insertOrderInvoice($allOrderDetail);
                                    $this->insertOrderInvoicePayment($allOrderDetail, $orderInvoiceId);
                                }*/
                $this->updateOrderPaymentState($allIdsorders[$i], $idStatusPaid);
            }
        }
    }

    /**
     * [updateOrderPaymentState updates payments states orders]
     * @param  [int] $idOrder      [ order id]
     * @param  [int] $idStatusPaid [is status payment]
     * @return [void]
     */
    private function updateOrderPaymentState($idOrder, $idStatusPaid)
    {
        require_once _PS_ROOT_DIR_ . '/config/config.inc.php';
        require_once _PS_CLASS_DIR_ . 'order/Order.php';
        require_once _PS_CLASS_DIR_ . 'order/OrderHistory.php';

        $objOrder = new Order($idOrder);

        if ($objOrder->hasBeenPaid()) {
            return 0;
        }

        $objOrder->setCurrentState($idStatusPaid); //order with id=1
    }
}
